/*
 * File originally named DemoWaveProbe.java
 * Created on 20. november 2007 by dfathi
 *
 * Modified greatly by Trygve Espeland Halvorsen
 * Spring 2008
 * 
 * And
 * 
 * Øyvind Auestad
 * Fall 2011
 * 
 * (FORMER KNOWN AS SESFEDERATE.JAVA
 */

package no.marintek.simulator.ses;

import no.marintek.csi.datadesk.IObjectStorage;
import no.marintek.simulator.environment.WavesModel;
import no.marintek.simulator.generic.GenericFederate;
import no.marintek.simulator.vessel.ShipMotionConversion;
import no.marintek.tools.dataobjects.Motion;
import no.marintek.tools.dataobjects.Ned;

public class AirCushion extends GenericFederate {
    
    // ----------------------------------------------------
    // Definitions
    // ----------------------------------------------------
    
   
    private double c_n;
    
    private double my = 0;

    private WavesModel Waves = new WavesModel();
    private Motion cgMotion;
    private Ned ned = new Ned();
             
    private boolean debugConfiguration = false;
    private double seastateRampTime = 0;
    private int n_fft = 0, n_active_HF = 200, n_active_LF = 200, tile=1, length = 500, n_patch=0;
    private boolean bUpdateSeastate=false;
    private double time, dt = 0;
    private boolean waves_initialized = false;
    private double lastUpdate = 0;
    
    private boolean initialized = false;

    private int last_spectrum;
    private double last_wave_height;
    private double last_wave_peak_period;
    private double last_wave_peakedness;
    private double last_wave_swell_direction;
    private double last_wave_wave_direction;
    private double last_wave_spreading_angle;
    
    private double cg_down; 
   
    private ShipMotionConversion convertShipMotion = new ShipMotionConversion();
    private Motion bowMotion;
    private double bow_down; // down from the NED frame at the bow of the SES.

    // AirCushion classes
    private Constants constant;             // Common constant data, obtained from the input class
    private AirCushionGeometry cushion;             // Geometry of the air cushion, local and global
   // private FanCharacteristic fan;          // Class containing fan characteristics
    //private CushionLeakage leakage;        
    private Seal sternSeal;                 // Geometry of stern Seal, local and global
    private Seal bowSeal;                   // Geometry of bow Seal, local and global
    private InitialSESData initData;        // Initial data
    private SESHull hull;                   // Geometry of vessel
    private SESMass mass;         
    private SESForces forces;
    private Calculations calc;
    private CushionLeakage leakageHull;
    private CushionVolume volume;
    private SurfaceEffectShipSeals SESseal;
    
    // Forces
    private double force_surge = 0;
    private double force_sway = 0;
    private double force_heave = 0;
    private double moment_roll = 0;
    private double moment_pitch = 0;
    private double moment_yaw = 0;
    private double force_x_rel_ap = 0;      // Point of attack of forces
    private double force_y_rel_cl = 0;
    private double force_z_rel_bl = 0;
    
    private double draught = 0;                 // Vessel's draught
    private double p_0 = 0;                     // Static air cushion over pressure
    private double p_c = 3500; // toDO                     // Air cushion over pressure
    private double p_c_prev = 0;                // Air cushion over pressure from last time step
    private double air_cushion_volume = 0;      // Enclosed air cushion volume
    private double air_cushion_volume_prev = 0;   // Enclosed air cushion volume from last time step
    private double wetSurface = 0;              // Wet surface of hull
    private double lift_ratio = 0;              // Air cushion lift / total vessel mass
    private double passive_gain;
   
    // Paramater settings
    private boolean cushion_active = false;         // The air cushion federate active or not
    private boolean update_air_cushion = true;      // True = update paramtersettings. Set true so that first thing federate will do is initialize itself
    private boolean on_cushion = false;    // Calculate forces due to air cushion over pressure
    public boolean forces_p_c = false;             // Calculate forces using air cushion overpressure. False: Use static over pressure
    private boolean fan_and_leak_active = false;    // Calculate air flow in/out from air cushion
    private boolean viscous_forces_active = false;         // Calculate added viscous resistance

    private String tempString;
    private String mgfPath;
    
    private double dVdt = 0;
    //Louver system
    private double Q_out = 0;   // Air flow out from the air cushion
    private double A_L_Dyn,sumTotalLeak = 0;
    private double A_L_Louvers = 0;
    private double A_L_Passive = 0;
    private double A_L_Mean_Louvers = 0;
    private double A_L_Dyn_Passive = 0;
    
    //double sixDOF[] = new double[6];
    
    //Fan system
    private double Q_in = 0;    // Air flow into the air cushion
    private double Q_0 = 0; // Static air flow
    private double Q_in_Dyn = 0;
    
    // To help decide RAO and mean wet surface
    double pitch_max = -1000;
    double pitch_min = 1000;
    double heave_max = -1000;
    double heave_min = 1000;
    double my_max = -1000;
    double my_min = 1000;
    
    double cg_surge_vel, cur_surge_vel = 0;

    double wetSurface_max = -1000;
    double wetSurface_min = 1000;
    double totalWetSurface = 0;
    int countWetSurface = 0;
    
    private Ned ned_wave_vel;
    
    public AirCushion(String configFile, String simType){
        super(configFile, simType);
        logger.info("Starting simulation");
        simulate();
    }
    
    public AirCushion(String configFile, String fedName, String simType){
        super(configFile, fedName, simType);
        logger.info("Starting simulation");
        simulate();
    }
        
    private void simulate(){  
        simApp.startSimulation();
        
        simApp.performStep();
       
        attributesReadOnce();
        
        
        while(simApp.isConnected()){
            performStep();
        }
        simApp.stopSimulation();
    } 
    
    private void performStep() {
       
        checkWaveAttributes();

        if (update_air_cushion || !initialized) {
            this.updateAirCushion();
           
        }

        if (this.cushion_active) {
            this.calculateAirCushionForces();
        } else {
            this.setForceZero();
        }
        updateSeaState();    
        simApp.performStep();
    }
    
    private void updateSeaState(){
        boolean timeToUpdateSeaState = (time- lastUpdate) > seastateRampTime;
        if (bUpdateSeastate && timeToUpdateSeaState) {
            bUpdateSeastate=false;
            logger.info("Air cushion model: Seastate updated");
        }
    }
    
    private void attributesReadOnce(){
        this.tempString = simApp.getModelProperties("unzip_folder",simApp.getFederateName()); // temp folder. Contains the vessel model from ShipX
        this.mgfPath = simApp.getModelProperties("mgf_path",simApp.getFederateName()); 
        
        
        // Create the constant class:
        this.constant = new Constants(
                1025,                                               // Mass density of water
                9.81,                                               // Acceleration of gravity
                simApp.getOwnAttribute("rho_a").getValueAsDouble(), // Mass density of air at atmospheric pressure
                simApp.getOwnAttribute("gamma").getValueAsDouble(), // Adiabatic constant
                simApp.getOwnAttribute("p_a").getValueAsDouble(),   // Atmospheric pressure
                simApp.getOwnAttribute("H").getValueAsDouble(),     // Roughness factor of hull surface
                simApp.getOwnAttribute("v").getValueAsDouble(),     // Kinematic viscosity
                simApp.getOwnAttribute("k").getValueAsDouble());    // Form factor
        
        // SES Seal
        this.SESseal = new SurfaceEffectShipSeals(constant.rho, constant.g, mgfPath, true); // TODO
        
         this.cushion = new AirCushionGeometry(mgfPath, 
                SESseal.getSternSealXforwardAP(),  // Aft extension of air cushion
                SESseal.getBowSealXforwardAP());  // Fore extension of air cushion
         

        this.sternSeal = new Seal(cushion, 
                simApp.getOwnAttribute("seal_stern_x_rel_ap").getValueAsDouble(), 
                simApp.getOwnAttribute("seal_stern_z_rel_bl").getValueAsDouble(), 0);
        this.bowSeal = new Seal(cushion, 
                simApp.getOwnAttribute("seal_bow_x_rel_ap").getValueAsDouble(), 
                simApp.getOwnAttribute("seal_bow_z_rel_bl").getValueAsDouble(), 1);

        this.hull = new SESHull(mgfPath);
        this.mass = new SESMass(tempString); // Find total weight of ship from retfun.re10 file (in vessel model archive)  
    //    System.out.println(SESseal.getSternSealXforwardAP() + " == " + simApp.getOwnAttribute("seal_stern_x_rel_ap").getValueAsDouble()
    //            + "AND" + SESseal.getBowSealXforwardAP() +" == " +simApp.getOwnAttribute("seal_bow_x_rel_ap").getValueAsDouble());
        
    }
    
    public void attributesReadEveryIteration(){
        this.force_x_rel_ap = simApp.getReceivedObjectAttribute("cg_x_rel_ap", "Case1_1.Vessel").getValueAsDouble();
        this.force_y_rel_cl = simApp.getReceivedObjectAttribute("cg_y_rel_cl", "Case1_1.Vessel").getValueAsDouble();
        this.force_z_rel_bl = simApp.getReceivedObjectAttribute("cg_z_rel_bl", "Case1_1.Vessel").getValueAsDouble();
        
        this.dt = simApp.getTimeStep();
        // Simulation time
        this.time = simApp.getReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0)).getValueAsDouble(); 
        this.draught = simApp.getReceivedObjectAttribute("loaded_draught_at_amidships").getValueAsDouble();

        
        ned.north = simApp.getReceivedObjectAttribute("cg_north","Case1_1.Vessel").getValueAsDouble();
        ned.east = simApp.getReceivedObjectAttribute("cg_east","Case1_1.Vessel").getValueAsDouble();
        ned.down = simApp.getReceivedObjectAttribute("cg_down","Case1_1.Vessel").getValueAsDouble();
        cg_down = ned.down;        //------------------
        
        cgMotion = new Motion();

        cgMotion.ned = ned;
        cgMotion.linearDisplacement.surge = ned.north;
        cgMotion.linearDisplacement.sway = ned.east;
        cgMotion.linearDisplacement.heave = ned.down;
        cgMotion.linearVelocity.surge = simApp.getReceivedObjectAttribute("cg_surge_vel").getValueAsDouble();
        cgMotion.linearVelocity.sway = simApp.getReceivedObjectAttribute("cg_sway_vel").getValueAsDouble();
        cgMotion.linearVelocity.heave = simApp.getReceivedObjectAttribute("cg_heave_vel").getValueAsDouble();
        cgMotion.linearAcceleration.surge = simApp.getReceivedObjectAttribute("cg_surge_acc").getValueAsDouble();
        cgMotion.linearAcceleration.sway = simApp.getReceivedObjectAttribute("cg_sway_acc").getValueAsDouble();
        cgMotion.linearAcceleration.heave = simApp.getReceivedObjectAttribute("cg_heave_acc").getValueAsDouble();
        cgMotion.angularDisplacement.roll = simApp.getReceivedObjectAttribute("roll").getValueAsDouble();
        cgMotion.angularDisplacement.pitch = simApp.getReceivedObjectAttribute("pitch").getValueAsDouble();
        cgMotion.angularDisplacement.yaw = simApp.getReceivedObjectAttribute("yaw").getValueAsDouble();
        cgMotion.angularVelocity.roll = simApp.getReceivedObjectAttribute("roll_vel").getValueAsDouble();
        cgMotion.angularVelocity.pitch = simApp.getReceivedObjectAttribute("pitch_vel").getValueAsDouble();
        cgMotion.angularVelocity.yaw = simApp.getReceivedObjectAttribute("yaw_vel").getValueAsDouble();
        cgMotion.angularAcceleration.roll = simApp.getReceivedObjectAttribute("roll_acc").getValueAsDouble();
        cgMotion.angularAcceleration.pitch = simApp.getReceivedObjectAttribute("pitch_acc").getValueAsDouble();
        cgMotion.angularAcceleration.yaw = simApp.getReceivedObjectAttribute("yaw_acc").getValueAsDouble();

        // tells the ship motion converter where cg is:
        convertShipMotion.setPositionOfCgRelToAPAndBL(
                simApp.getReceivedObjectAttribute("cg_x_rel_ap").getValueAsDouble(),
                simApp.getReceivedObjectAttribute("cg_y_rel_cl").getValueAsDouble(),
                simApp.getReceivedObjectAttribute("cg_z_rel_bl").getValueAsDouble());
        // Update global cushion and hull coordinates and submergence
        cushion.updateGlobal(cgMotion, convertShipMotion, constant, p_0, time);
        hull.updateGlobal(cgMotion, convertShipMotion, time); 
       
        //BEFORE THE NEW SEALS
        //sternSeal.updateGlobal(cgMotion, convertShipMotion);
        //bowSeal.updateGlobal(cgMotion, convertShipMotion);        
        
        // Get heave position at the bow of the SES (for ride controller):
        //convertShipMotion.getMotionAtPoint
        bowMotion = new Motion();
        bowMotion = convertShipMotion.getMotionAtPoint( 30,0,5, cgMotion); // TODO x bow rel ap...
        bow_down = bowMotion.ned.down;

        simApp.getOwnAttribute("A_L_Dyn_Passive").setValueAsDouble(A_L_Dyn_Passive);
        update_air_cushion = simApp.getOwnParameter("update_air_cushion").getValueAsBoolean();
        simApp.getOwnAttribute("update_air_cushion").setValueAsBoolean(update_air_cushion);

        // Leakage from louvers
        this.A_L_Louvers = simApp.getReceivedObjectAttribute("A_L_Louvers","Case1_1.Louver").getValueAsDouble();
                                
        if (viscous_forces_active){
            // For viscous calculations:
            cg_surge_vel = simApp.getReceivedObjectAttribute("cg_surge_vel", "Case1_1.Vessel").getValueAsDouble();
            cur_surge_vel = simApp.getReceivedObjectAttribute("cur_surge_vel", "Case1_1.Vessel").getValueAsDouble();
            
            ned_wave_vel = new Ned();
            ned_wave_vel.north = simApp.getReceivedObjectAttribute("cb_wave_vel_x","Case1_1.Wave").getValueAsDouble(); // Obtain particle velocity from wave federate
            ned_wave_vel.east = simApp.getReceivedObjectAttribute("cb_wave_vel_y","Case1_1.Wave").getValueAsDouble();  // Obtain particle velocity from wave federate
            ned_wave_vel.down = simApp.getReceivedObjectAttribute("cb_wave_vel_z","Case1_1.Wave").getValueAsDouble();  // Obtain particle velocity from wave federate
        } 
        
        passive_gain = simApp.getOwnParameter("passive_gain").getValueAsDouble();
    }
    
    private void attributesReadOnUpdate(){
        // Get Q_0 (eq. air flow)
        Q_0 = simApp.getReceivedObjectAttribute("Q_0","Case1_1.Fan").getValueAsDouble();            
        simApp.getOwnAttribute("Q_0").setValueAsDouble(Q_0);
        
        this.p_0 = simApp.getOwnParameter("p_0").getValueAsDouble();
        simApp.getOwnAttribute("p_0").setValueAsDouble(p_0);
        
        this.cushion_active = simApp.getOwnParameter("cushion_active").getValueAsBoolean();  
        this.on_cushion = simApp.getOwnParameter("on_cushion").getValueAsBoolean(); 
        this.forces_p_c = simApp.getOwnParameter("forces_p_c").getValueAsBoolean();
        this.fan_and_leak_active = simApp.getOwnParameter("fan_and_leak_active").getValueAsBoolean();
        this.viscous_forces_active = simApp.getOwnParameter("viscous_forces").getValueAsBoolean();
        this.dt = simApp.getTimeStep();
        //Get mean leakage:
        this.A_L_Mean_Louvers = simApp.getReceivedObjectAttribute("A_L_Mean_Louvers","Case1_1.Louver").getValueAsDouble();
    }
    
    private void attributesUpdateEveryIteration(){
        
        simApp.getOwnAttribute("lift_ratio").setValueAsDouble(lift_ratio);

        // Send forces to federate attributes
        simApp.getOwnAttribute("cushion_force_surge").setValueAsDouble(forces.getCushionSurge() + SESseal.getSurgeForce());
        simApp.getOwnAttribute("cushion_force_sway").setValueAsDouble(forces.getCushionSway());
        simApp.getOwnAttribute("cushion_force_heave").setValueAsDouble(forces.getCushionHeave());
        simApp.getOwnAttribute("cushion_moment_roll").setValueAsDouble(forces.getCushionRoll());
        simApp.getOwnAttribute("cushion_moment_pitch").setValueAsDouble(forces.getCushionPitch() + SESseal.getPitchMoment());
        simApp.getOwnAttribute("cushion_moment_yaw").setValueAsDouble(forces.getCushionYaw());


        // Send forces to federate attributes
        simApp.getOwnAttribute("viscous_force_surge").setValueAsDouble(forces.getTotalViscousSurge());
        simApp.getOwnAttribute("viscous_moment_pitch").setValueAsDouble(forces.getTotalViscousPitch());
        simApp.getOwnAttribute("viscous_calm_force_surge").setValueAsDouble(forces.getCalmViscousSurge());
        simApp.getOwnAttribute("viscous_calm_moment_pitch").setValueAsDouble(forces.getCalmViscousPitch());
        simApp.getOwnAttribute("viscous_added_force_surge").setValueAsDouble(forces.getAddedViscousSurge());
        simApp.getOwnAttribute("viscous_added_moment_pitch").setValueAsDouble(forces.getAddedViscousPitch());


        // ----------------------------------------------------
        // Update own attributes
        // ---------------------------------------------------- 
        simApp.getOwnAttribute("cg_down").setValueAsDouble(cg_down);
        simApp.getOwnAttribute("bow_down").setValueAsDouble(bow_down);
        simApp.getOwnAttribute("force_surge").setValueAsDouble(force_surge);
        simApp.getOwnAttribute("force_sway").setValueAsDouble(force_sway);
        simApp.getOwnAttribute("force_heave").setValueAsDouble(force_heave);
        simApp.getOwnAttribute("moment_roll").setValueAsDouble(moment_roll);
        simApp.getOwnAttribute("moment_pitch").setValueAsDouble(moment_pitch);
        simApp.getOwnAttribute("moment_yaw").setValueAsDouble(moment_yaw);
        simApp.getOwnAttribute("x_rel_ap").setValueAsDouble(force_x_rel_ap);
        simApp.getOwnAttribute("y_rel_cl").setValueAsDouble(force_y_rel_cl);
        simApp.getOwnAttribute("z_rel_bl").setValueAsDouble(force_z_rel_bl);
        simApp.getOwnAttribute("p_c").setValueAsDouble(p_c);
        simApp.getOwnAttribute("wet_surface").setValueAsDouble(wetSurface);
        simApp.getOwnAttribute("wet_surface_0").setValueAsDouble(initData.getwet_surface_0());
        simApp.getOwnAttribute("cushion_volume").setValueAsDouble(air_cushion_volume);
        simApp.getOwnAttribute("cushion_volume_0").setValueAsDouble(initData.getcushion_volume_0());
        simApp.getOwnAttribute("Q_in").setValueAsDouble(Q_in);
        simApp.getOwnAttribute("Q_out").setValueAsDouble(Q_out);
        simApp.getOwnAttribute("mu").setValueAsDouble(my);
        simApp.getOwnAttribute("lift_ratio_0").setValueAsDouble(initData.getLift_ratio_0());   
        simApp.getOwnAttribute("c_n").setValueAsDouble(c_n);
        
        
        
    }
    
    private void updateAirCushion(){
        

        // Update received attributes from federate
        attributesReadOnUpdate();
        
        System.out.println("AirCushion: Updating air cushion files and paramaters, p_0 now equal: " + p_0);
        
        if (this.cushion_active){
            
            if(Q_0 > 0 && time > 0.2){
                initialized = true;
            }
            
            update_air_cushion = false;
            simApp.getOwnParameter("update_air_cushion").setValueAsBoolean(update_air_cushion);
            simApp.getOwnAttribute("update_air_cushion").setValueAsBoolean(update_air_cushion);
            volume = new CushionVolume();
            leakageHull = new CushionLeakage(cushion, hull, p_0, constant, draught);
            A_L_Passive = leakageHull.getHullLeakageStatic();

            // Static calculations
            initData = new InitialSESData(hull, cushion, sternSeal, bowSeal, 
                    p_0, draught, mass, constant, volume, SESseal);  

            air_cushion_volume = initData.getcushion_volume_0();
            air_cushion_volume_prev = air_cushion_volume;

            p_c_prev = p_0;


            A_L_Dyn = 0;
            dVdt = 0;

            calc = new Calculations();
            
            calc.updateCalculationData(constant, initData, p_0, air_cushion_volume, 
                     0, A_L_Mean_Louvers, A_L_Mean_Louvers, 0, dt ,Q_0, Q_0);
            calc.getMy(fan_and_leak_active);
            calc.getMy(fan_and_leak_active);

            forces = new SESForces(cushion, p_c);            
        }
        
    }

    private void calculateAirCushionForces() {
        
        // Main loop when cushion is on and active

        if (!waves_initialized) return;
        
        try {

            attributesReadEveryIteration();
            
            //c_n = simApp.getOwnParameter("c_n").getValueAsDouble();
            
            volume.calculateVolume(cushion, p_0, constant, time);
            this.air_cushion_volume = volume.getCushionVolume() + SESseal.getBowCushionVolume();
            
            // Calculate enclosed air cushion volume and wet surface.
            WetHull wetsurface = new WetHull(hull, cushion);
            this.wetSurface = wetsurface.wetArea;
                        
            // Calculate leakages due to hull and seals (passive leakage)
          //  leakage = new CushionLeakage(cushion, sternSeal, bowSeal, hull, p_0, constant, time);
            //A_L_Passive = leakage.totalPassiveLeakage;
            
            // NEW SEAL STARTS HERE-----------------------------------
            SESseal.calcSealForceAndLeakageArea(time, p_c, ned.north, ned.east, ned.down, cgMotion.angularDisplacement.roll, cgMotion.angularDisplacement.pitch,cgMotion.angularDisplacement.yaw, Waves);
            
            
            double [] sternSealForce = new double[6];
            double [] bowSealForce =  new double[6];
            
            SESseal.getBowForce(bowSealForce);
            SESseal.getSternForce(sternSealForce);
                                     
            // Implementation of seal forces and leak:
            A_L_Passive = passive_gain * (SESseal.getBottomLeakageArea() + SESseal.getSideLeakageArea()) + leakageHull.getHullLeakage();
            
            // Fan contribution
            Q_in = simApp.getReceivedObjectAttribute("Q_in","Case1_1.Fan").getValueAsDouble();
            Q_in_Dyn = Q_in - Q_0;
            
            //Solve differential equation for air cushion volume
            dVdt = (air_cushion_volume-air_cushion_volume_prev)/dt;

            // Find uniform air cushion over pressure
            calc.updateCalculationData(constant, initData, p_0, air_cushion_volume, A_L_Passive, A_L_Louvers, A_L_Mean_Louvers, dVdt, dt, Q_0, Q_in);
                        
            my = calc.getMy(fan_and_leak_active);
          //  System.out.println("my = "+my);
            p_c = calc.getPc(my);
            c_n = calc.getCn();
            //System.out.println("AirCushion: Q_in = "+Q_in+"p_c = "+p_c);
            Q_out = calc.getQout(fan_and_leak_active);

            // Update forces, reading new parameters and calculates new forces
            forces.updateForces(on_cushion, viscous_forces_active, 
            initData, p_c, cushion, convertShipMotion, ned_wave_vel, hull, draught, 
            cg_surge_vel, cur_surge_vel, constant, wetSurface, cgMotion);
            
            // Use only cushion forces. Only send these to Vessel federate
            force_surge = forces.getCushionSurge();
            force_sway = forces.getCushionSway();
            force_heave = forces.getCushionHeave();
            moment_roll = forces.getCushionRoll();
            moment_pitch = forces.getCushionPitch() ;
            moment_yaw = forces.getCushionYaw();
            
            // Send all forces acting on the cushion
            /*
            force_surge = forces.getCushionSurge() + SESseal.getSurgeForce() + forces.getTotalViscousSurge();
            force_sway = forces.getCushionSway() + SESseal.getSwayForce();
            force_heave = forces.getCushionHeave() + SESseal.getHeaveForce();
            moment_roll = forces.getCushionRoll() + SESseal.getRollMoment();
            moment_pitch = forces.getCushionPitch()  + SESseal.getPitchMoment() + forces.getTotalViscousPitch();;
            moment_yaw = forces.getCushionYaw() + SESseal.getYawMoment();
            */

            lift_ratio = (forces.cforce_heave)*100/(mass.massTotal*constant.g);
             
            // Store data for next run
            p_c_prev = p_c;
            air_cushion_volume_prev = air_cushion_volume;
            
            // Update federate attributes to the CSI bus
            attributesUpdateEveryIteration();
            
            
        } catch(Exception e) {
            logger.severe("Could not calculate air cushion forces. Attempting to update air cushion paramaters.");
            simApp.getOwnParameter("update_air_cushion").setValueAsBoolean(true);
        }
    }
      
    private void setForceZero() {
        // Set all forces equals zero
        simApp.getOwnAttribute("force_surge").setValueAsDouble(0);
        simApp.getOwnAttribute("force_sway").setValueAsDouble(0);
        simApp.getOwnAttribute("force_heave").setValueAsDouble(0);
        simApp.getOwnAttribute("moment_roll").setValueAsDouble(0);
        simApp.getOwnAttribute("moment_pitch").setValueAsDouble(0);
        simApp.getOwnAttribute("moment_yaw").setValueAsDouble(0);  
        if (update_air_cushion == true) {     
                update_air_cushion = false;
                simApp.getOwnParameter("update_air_cushion").setValueAsBoolean(update_air_cushion);
                simApp.getOwnAttribute("update_air_cushion").setValueAsBoolean(update_air_cushion);
        }
    }
    
    private void checkWaveAttributes() {
        int iErrPos=0;
      
        try{
          
          if (simApp.getObjectNamesOfClass("Time").size()<1) return;
          if(!simApp.containsReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0))) return;
          
          IObjectStorage object = (IObjectStorage)simApp.getOwnObjectList().get(0);
          
          // TODO HERE
          //String wave = object.getObjectName().substring(0, object.getObjectName().indexOf("."))+".Wave";
          String wave = "Case1_1.Wave";
                
          if(!simApp.containsReceivedObjectAttribute("spectrum", wave)) return;

          int current_spectrum=simApp.getReceivedObjectAttribute("spectrum",wave).getValueAsInteger();
          //if (debugConfiguration) System.out.println("current_spectrum " + current_spectrum);
          double current_wave_height=simApp.getReceivedObjectAttribute("wave_height",wave).getValueAsDouble();
          //if (debugConfiguration) System.out.println("current_wave_height " + current_wave_height);
          double current_wave_peak_period=simApp.getReceivedObjectAttribute("peak_period",wave).getValueAsDouble();
          double current_wave_peakedness=simApp.getReceivedObjectAttribute("peakedness",wave).getValueAsDouble();
          double current_wave_swell_direction=simApp.getReceivedObjectAttribute("swell_direction",wave).getValueAsDouble();
          double current_wave_wave_direction=simApp.getReceivedObjectAttribute("wave_direction",wave).getValueAsDouble();
          double current_wave_spreading_angle=simApp.getReceivedObjectAttribute("spreading_angle",wave).getValueAsDouble();

          iErrPos=1;
          
          if (!waves_initialized && current_wave_peak_period>0.0) {
            
            logger.info("PropulsionDriver Wave model: Initializing new seastate");
            
            if (debugConfiguration) System.out.println("current_spectrum " + current_spectrum);
            if (debugConfiguration) System.out.println("current_wave_height " + current_wave_height);
            if (debugConfiguration) System.out.println("current_wave_peak_period " + current_wave_peak_period);
            if (debugConfiguration) System.out.println("current_wave_peakedness " + current_wave_peakedness);
            if (debugConfiguration) System.out.println("current_wave_spreading_angle " + current_wave_spreading_angle);
      
            n_fft = simApp.getReceivedObjectAttribute("n_fft",wave).getValueAsInteger();
            if (debugConfiguration) System.out.println("n_fft " + n_fft);
            
            n_active_HF = simApp.getReceivedObjectAttribute("n_active_HF",wave).getValueAsInteger();
            if (debugConfiguration) System.out.println("n_active_HF " + n_active_HF);
            
            n_active_LF =simApp.getReceivedObjectAttribute("n_active_LF",wave).getValueAsInteger();
            if (debugConfiguration) System.out.println("n_active_LF " + n_active_LF);
            
            seastateRampTime = simApp.getReceivedObjectAttribute("seastate_ramp_time",wave).getValueAsDouble();
            if (debugConfiguration) System.out.println("seastate_ramp_time " + seastateRampTime);
            
            length = simApp.getReceivedObjectAttribute("length",wave).getValueAsInteger();
            if (debugConfiguration) System.out.println("length " + length);
            
            tile = simApp.getReceivedObjectAttribute("tile",wave).getValueAsInteger();
            if (debugConfiguration) System.out.println("tile " + tile);

            if (tile==0){ 
              n_patch = n_fft; 
            } else {
              n_patch = n_fft + 1;
            }

            if (debugConfiguration) System.out.println("time " + simApp.getReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0)).getValueAsDouble());

            iErrPos=2;
            Waves.freeWaveMemory();
            
            Waves.initSeastateData(n_fft, n_active_HF, n_active_LF, tile, length, seastateRampTime);
                       
            if (current_spectrum==-1){
              String waveTSWFile= simApp.getReceivedObjectAttribute("tsw_file",wave).getValueAsString();
              if (debugConfiguration) System.out.println("waveTSWFile " + waveTSWFile);
              Waves.setWaveComponentsData(waveTSWFile);
            }

            Waves.initSeastate(current_spectrum,
                  current_wave_height,
                  current_wave_peak_period,
                  current_wave_peakedness,
                  current_wave_swell_direction,
                  current_wave_wave_direction,
                  current_wave_spreading_angle,
                  simApp.getReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0)).getValueAsDouble());
            
            //System.out.println("juhuhu");
            iErrPos=3;
            //System.out.println("juhu time "+simApp.getReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0)).getValueAsDouble());
            
            last_spectrum=current_spectrum;
            last_wave_height=current_wave_height;
            last_wave_peak_period=current_wave_peak_period;
            last_wave_peakedness=current_wave_peakedness;
            last_wave_swell_direction=current_wave_swell_direction;
            last_wave_wave_direction=current_wave_wave_direction;
            last_wave_spreading_angle=current_wave_spreading_angle;
            

            waves_initialized = true;
          } else {
            // Check if seastate has changed...
            if (last_spectrum!=current_spectrum ||
                last_wave_height!=current_wave_height ||
                last_wave_peak_period!=current_wave_peak_period ||
                last_wave_peakedness!=current_wave_peakedness ||
                last_wave_swell_direction!=current_wave_swell_direction ||
                last_wave_wave_direction!=current_wave_wave_direction ||
                last_wave_spreading_angle!=current_wave_spreading_angle){
                  bUpdateSeastate=true;
                  iErrPos=4;
                }
          }
          
          if (bUpdateSeastate &&
              ((time - lastUpdate) > seastateRampTime)) {
            
            lastUpdate=time;
            try {

              logger.info("PropulsionDriver Wave model: Updating seastate");
              
              if (debugConfiguration) System.out.println("current_spectrum " + current_spectrum);
              if (debugConfiguration) System.out.println("current_wave_height " + current_wave_height);
              if (debugConfiguration) System.out.println("current_wave_peak_period " + current_wave_peak_period);
              if (debugConfiguration) System.out.println("current_wave_peakedness " + current_wave_peakedness);
              if (debugConfiguration) System.out.println("current_wave_spreading_angle " + current_wave_spreading_angle);

              if (current_spectrum==-1){
                String waveTSWFile= simApp.getReceivedObjectAttribute("tsw_file",wave).getValueAsString();
                if (debugConfiguration) System.out.println("waveTSWFile " + waveTSWFile);
                Waves.setWaveComponentsData(waveTSWFile);
              }
              
              iErrPos=5;
              Waves.initSeastate(current_spectrum,
                  current_wave_height,
                  current_wave_peak_period,
                  current_wave_peakedness,
                  current_wave_swell_direction,
                  current_wave_wave_direction,
                  current_wave_spreading_angle,
                  simApp.getReceivedObjectAttribute("time", simApp.getObjectNamesOfClass("Time").get(0)).getValueAsDouble());
              iErrPos=6; 
              last_spectrum=current_spectrum;
              last_wave_height=current_wave_height;
              last_wave_peak_period=current_wave_peak_period;
              last_wave_peakedness=current_wave_peakedness;
              last_wave_swell_direction=current_wave_swell_direction;
              last_wave_wave_direction=current_wave_wave_direction;
              last_wave_spreading_angle=current_wave_spreading_angle;

            } catch (Exception ex) {
              logger.info("PropulsionDriver Wave model: Problem parsing wave attributes when updating seastate...");
            }
          }

  
        } catch(Exception e){
            logger.severe("Could not get Wave Model attributes. iErrPos= "+iErrPos+" n_patch = "+n_patch+" Error description: "+e.toString());
        }
    }
       
    
    public static void main(String[] args){
        // This class needs input from the command line to start correctly
        String federate = null;         // First input argument
        String simBus = null;           // Second input argument
        String federationFile = null;   // Third input argument
               
        if(args.length > 1){
            federationFile = args[0];
            if(args[1].equalsIgnoreCase("csi") || args[1].equalsIgnoreCase("hla")){
                simBus = args[1];
                System.out.println(simBus);
                AirCushion SESvessel = new AirCushion(federationFile, simBus);
            } else {
                federate = args[1];
            }
        } else{
            System.out.println("Federation file and sim bus type must be specified");
        }
        
        if(args.length > 2){
            federate = args[1];
            simBus = args[2];
            AirCushion SESvessel = new AirCushion(federationFile,federate, simBus);
        }
    }
}