/*
 * Calculations
 * This class performs different calculations that are
 * either needed initially or calculations needed every 
 * iteration.
 * @author Trygve Espeland & Øyvind Auestad
 */
package no.marintek.simulator.ses;

public class Calculations {
    private Constants constant;
    private double c_n;
    
    double A = 0;
    double lastA = 0;
    double B = 0;
    double lastB = 0;
    
    // A saturation limit restricting how much "my" can change for each iteration
    // This is for slowing down the dynamics and ruling out unrealistic (quick) cushion changes
    // double sat_limit = 100;
    
    private InitialSESData initData;
    private double p_0;
    private double air_cushion_volume;
    private double sumTotalLeak;
    private double Q_in_Dyn;
    private double A_L_Dyn;
    private double A_L_Mean_Louvers;
    private double dVdt;
    private double dt;
    private double Q_0;
    private double Q_in;
    
    private double my;
    private double prev_my;
    private double p = 0;
    private double lastp = 0;
    
    private double p_c;
    
    private double Q_out;
    

    
    public Calculations(){
        // Constructor
    }
    
    public void updateCalculationData(Constants constant, InitialSESData initData, double p_0, double air_cushion_volume, 
            double A_L_Passive, double A_L_Louvers, double A_L_Mean_Louvers, double dVdt, double dt, double Q_0, double Q_in){
    
        this.p_0 = p_0;
        this.air_cushion_volume = air_cushion_volume;
        this.sumTotalLeak = A_L_Passive + A_L_Louvers;
        this.Q_in_Dyn = Q_in - Q_0;
        this.A_L_Mean_Louvers = A_L_Mean_Louvers;
        this.A_L_Dyn = A_L_Louvers - A_L_Mean_Louvers; // Passive Dyn is neglectable (so small)
        this.dVdt = dVdt;
        this.dt = dt;
        this.initData = initData;
        this.constant = constant;
        this.Q_0 = Q_0;
        this.Q_in = Q_in;
        calculateContraction();
    }
    
    private void calculateContraction(){
        c_n = Q_0 / (A_L_Mean_Louvers * Math.sqrt(2*p_0/constant.rho_a));
    }
    
    public double getCn(){
        return c_n;
    }
    
    
    
    public double getMy(boolean fan_and_leak_active){
        double hereWasRhoA = 1;
        
        //System.out.println("volume = "+air_cushion_volume);

        if (fan_and_leak_active){
            my = 0;
            // Integration variables:
            A = (constant.gamma/hereWasRhoA)*((p_0+constant.p_a)/p_0)*(1/air_cushion_volume)*
            c_n*(sumTotalLeak)*Math.sqrt(2*p_0/constant.rho_a)*0.5
            + (1/air_cushion_volume)*dVdt;

            B = (constant.gamma/hereWasRhoA)*((p_0+constant.p_a)/p_0)*(1/air_cushion_volume)*
            (Q_in_Dyn-c_n*A_L_Dyn*Math.sqrt(2*p_0/constant.rho_a)-constant.rho_a*dVdt);

            p = 0.5*(lastA+A)*dt;
            my = Math.exp(-p)*0.5*(lastB*Math.exp(lastp) + B*Math.exp(p))*dt;

            // Store data for next time step
            lastA = A;
            lastB = B;
            lastp = p;  
        }
        else{
            my = ((p_0 + constant.p_a)/p_0)
            *(Math.pow(initData.getcushion_volume_0()/air_cushion_volume, constant.gamma)-1);
        }
        /*
        // Saturation limit on how quick the cushion can change (slow the dynamics)
        if (my > prev_my + sat_limit){
        my = prev_my + sat_limit;
        }
        else if(my < prev_my - sat_limit){
        my = prev_my - sat_limit;
        }
        prev_my = my;  
         * 
         */
        return my;
    }
    
    public double getPc(double my){
        
        p_c = p_0*(1+my);
        return p_c;
    }
    
    public double getQout(boolean fan_and_leak_active){
        
        if (fan_and_leak_active){
          // System.out.println(constant.c_n +"  "+ sumTotalLeak+"   "+ p_c+"    "+constant.rho_a); 
          //  Q_out = constant.c_n * sumTotalLeak * Math.sqrt((2*p_c)/constant.rho_a);
           // System.out.println(Q_out);
            Q_out = Q_0+c_n*A_L_Dyn*Math.sqrt(2*p_0/constant.rho_a)
            +c_n*(sumTotalLeak)*Math.sqrt(2*p_0/constant.rho_a)*0.5*my;
        }
        else{
            Q_out = 0;
        }
        return Q_out;
    }        
    
    public double getPcNew(){
        
        // Important notation. Doing taylor expansion
        // d(Pc)/dt = F
        // d(F)/dPc (p_0) = dFd(p_c)_p_0 jesus....
        dVdt = 0;
       
        double ga = constant.gamma;
        double p_a = constant.p_a;
        double tmp = (ga-1)/ga;
        double tmp2 = -1/ga;
        
        double c1 = Math.pow((p_a+p_0),-tmp2);
        double c2 = c_n * sumTotalLeak*Math.sqrt((2/constant.rho_a));

        System.out.println(p_a +" - "+ ga  +" - "+ sumTotalLeak +" - "+ c_n +" - "+ air_cushion_volume +" - "+ p_0  +" - "+  dVdt);
        
        double c3 = (ga/air_cushion_volume) * Math.pow((p_0+p_a),tmp) * ( c1*(Q_in - c2*Math.sqrt(p_0)) - c1 * dVdt);
        double c4 = c3*tmp*Math.pow((p_0 + p_a),-1)
                    + (ga/air_cushion_volume) * Math.pow((p_0+p_a),tmp) * (-c1*c2*0.5*Math.pow(p_0,-0.5) - c1*dVdt*(1/ga)*Math.pow((p_a+p_0),-tmp));
        
        A = -c4;
        B = c3 - (c4*p_0);
        
        
        
        p = 0.5*(lastA+A)*dt;
        double p_c_new = Math.exp(-p)*0.5*(lastB*Math.exp(lastp) + B*Math.exp(p))*dt;
        

        // Store data for next time step
        lastA = A;
        lastB = B;
        lastp = p;  

        //System.out.println("p_c !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!= "+p_c_new);
        
        return p_c_new;
        
    }
    
    public double getMyNew(){
        return ((p_c/p_0) - 1);
    }
    
    public double getQoutNew(){
        double Qout = c_n*sumTotalLeak*Math.sqrt((2*p_c/constant.rho_a));
        return Qout;
    }
}