/*
 * CushionLeakage
 * Calculates leakages from air cushion
 * Two main differences:
 * Leakages from under hull
 * Leakages from skirt
 * 
 * Made by Trygve Espeland Halvorsen & Øyvind Auestad, Spring 2008 / Summer 2011
 * ONLY HULL
 */


package no.marintek.simulator.ses;

import no.marintek.simulator.environment.WavesModel;

public class CushionLeakage {
    public double totalPassiveLeakage = 0;
    public double sternLeakage = 0;
    public double bowLeakage = 0;
    private AirCushionGeometry cushion;   
    private double p_c;
    private double p_0;
    private Constants constant;
    private SESHull hull;
    private double draught;
    public double hullLeakage = 0;
    private double hullLeakageStatic;
    private double time;
    
    // Instantaneous calculations
    public CushionLeakage(AirCushionGeometry cushion, SESHull hull, double p_0, Constants constant, double draught) {
        this.cushion = cushion;
        this.hull = hull;
        this.constant = constant;
        this.draught = draught;
        this.p_0 = p_0;
        
        this.leakageHullStatic();
    }
    
    public void calculateHullLeakage(double p_c, double time, WavesModel waves){
        this.p_c = p_c;
        this.time = time;
        
        double leakage = 0;
        double pressureWC = this.p_c/(constant.rho*constant.g);
        
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++) {
            
            // ----- Starboard side -----
            if (cushion.firstCoordInAirStarb[secNr] == 0) {
                double waveHeight = waves.getInterpolatedWaveElevation(time,cushion.globalStarBxCoord[secNr][0],cushion.globalStarByCoord[secNr][0]);
                double hullHeight = cushion.globalStarBzCoord[secNr][0];
                double submergence = -hullHeight + Math.max(0, pressureWC) + waveHeight;
                // Skjønner ikke denne minimum problemet.
                double leakeDistance = Math.min(submergence, cushion.localyCoord[secNr][0]);
                leakage += leakeDistance*cushion.globaldL[secNr];
            }

            // ----- Aport side -----
            if (cushion.firstCoordInAirAPort[secNr] == 0) {
                double waveHeight = waves.getInterpolatedWaveElevation(time,cushion.globalAPortxCoord[secNr][0],cushion.globalAPortyCoord[secNr][0]);
                double hullHeight = cushion.globalAPortzCoord[secNr][0];
                double submergence = -hullHeight + Math.max(0, pressureWC) + waveHeight;
                double leakeDistance = Math.min(submergence, cushion.localyCoord[secNr][0]);
                leakage += leakeDistance*cushion.globaldL[secNr];
            }
        }
        hullLeakage = leakage;
        
    }
    
       
    private void leakageHullStatic() {
        double leakage = 0;
        double pressureWC = this.p_0/(constant.rho*constant.g); 
      
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++) {
            
            // ----- Starboard side -----
            if (cushion.firstCoordInAirStarb[secNr] == 0) {
                double submergence = cushion.localzCoord[secNr][0] - draught + pressureWC;
                double leakeDistance = Math.min(submergence, cushion.localyCoord[secNr][0]);
                leakage += leakeDistance*cushion.globaldL[secNr];
            }
        }
        
        // Aport side
        // perfect symmetry
        this.hullLeakageStatic = 2 * leakage;
    }
    
    public double getHullLeakage(){
        return this.hullLeakage;
    }
    
    public double getHullLeakageStatic(){
        return this.hullLeakageStatic;
    }
    
  
}