/*
 * CushionVolume
 * Calculates enclosed air cushion volume
 * Two constructors:
 * One for static condition and one for instantaneous
 * 
 * Made by Trygve Espeland Halvorsen, Spring 2008
 */

package no.marintek.simulator.ses;

import no.marintek.simulator.environment.WavesModel;


public class CushionVolume {
    private double minDistance = 1; // Coarseness of instantaneous volume calculations
    private double airCushionVolume = 0;
    private double prevAirCushionVolume = 0;
    private double volumeSaterationPerIteration = 1000000;  //19
    
    public void calculateVolume(AirCushionGeometry cushion, double p_c , Constants constant, double time) {
        //volume.calculateVolume(cushion, p_0, constant, time);
        
        WavesModel waves = new WavesModel();
        
        this.airCushionVolume = 0;
        double pressureWC = p_c/(constant.rho*constant.g);
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++){
            double sectionArea = 0;

            // ----- Starboard side ----- 
            boolean goon = true;
            
            double nextx = 0;
            double nexty = 0;
            double nextz = 0;
            double lastx = cushion.crossxCoordGlobalStarB[secNr];
            double lasty = cushion.crossyCoordGlobalStarB[secNr];
            double lastz = cushion.crosszCoordGlobalStarB[secNr];
            int nextCoord = cushion.firstCoordInAirStarb[secNr];
            double addOrSubtract = 1;
            // Information about the orientation of the hull surface is lost during the calculations
            // "addOrSubtract" keeps track of whether calculated area should be added or subtracted

            double wave_elevation_xy = waves.getInterpolatedWaveElevation(time,lastx,lasty) // pos.down
                + pressureWC; // pos. down
            double previousSubmergence = -lastz+wave_elevation_xy; // Positive means above water
            double submergence = 0;
            while (goon) {
                nextx = cushion.globalStarBxCoord[secNr][nextCoord];
                nexty = cushion.globalStarByCoord[secNr][nextCoord];
                nextz = cushion.globalStarBzCoord[secNr][nextCoord];

                double distance = Math.sqrt((lastx-nextx)*(lastx-nextx) + (lasty-nexty)*(lasty-nexty));
                // Distance in xy plane

                double vector[] = new double[3];
                vector[0] = nextx-lastx;
                vector[1] = nexty-lasty;
                vector[2] = nextz-lastz;
                double zDirection = vector[0]*cushion.globalShipDirectionVector[1] - vector[1]*cushion.globalShipDirectionVector[0];
                
                if (Math.abs(zDirection) < 0.01) {
                    addOrSubtract = 0;
                } else if (zDirection < 0) {
                    addOrSubtract = -1;
                } else {
                    addOrSubtract = 1;
                }
                    
                if (distance < minDistance) {
                    wave_elevation_xy = waves.getInterpolatedWaveElevation(time,nextx,nexty) // pos.down
                            + pressureWC; // pos. down
                    submergence = -nextz+wave_elevation_xy; // Positive means above water
                    sectionArea += addOrSubtract * distance * (submergence+previousSubmergence)/2;
                } else {
                    double steps = Math.ceil(distance/minDistance);
                    double xStep = (nextx-lastx)/steps;
                    double yStep = (nexty-lasty)/steps;
                    double zStep = (nextz-lastz)/steps;
                    
                    for (int i=1; i<=steps; i++) {
                        nextx = lastx+xStep*i;
                        nexty = lasty+yStep*i;
                        nextz = lastz+zStep*i; 
                        wave_elevation_xy = waves.getInterpolatedWaveElevation(time,nextx,nexty) // pos.down
                                + pressureWC; // pos. down
                        submergence = -nextz+wave_elevation_xy; // Positive means above water
                        sectionArea += addOrSubtract * Math.sqrt(xStep*xStep+yStep*yStep) * (submergence+previousSubmergence)/2;
                        previousSubmergence = submergence;
                    }
                }
                
                if (nextCoord == cushion.coordNumber[secNr]-1) {
                    goon = false; }
                lastx = nextx;
                lasty = nexty;
                lastz = nextz;
                previousSubmergence = submergence;
                nextCoord++;
            }  
            
            
            // ----- Aport side ----- 
            goon = true;
            lastx = cushion.crossxCoordGlobalAPort[secNr];
            lasty = cushion.crossyCoordGlobalAPort[secNr];
            lastz = cushion.crosszCoordGlobalAPort[secNr]; 
            nextCoord = cushion.firstCoordInAirStarb[secNr];
            
            wave_elevation_xy = waves.getInterpolatedWaveElevation(time,lastx,lasty)
                    + pressureWC; // pos. down 
            previousSubmergence = -lastz+wave_elevation_xy; // Positive means above water
            
            while (goon) {
                nextx = cushion.globalAPortxCoord[secNr][nextCoord];
                nexty = cushion.globalAPortyCoord[secNr][nextCoord];
                nextz = cushion.globalAPortzCoord[secNr][nextCoord];
                double distance = Math.sqrt((lastx-nextx)*(lastx-nextx) + (lasty-nexty)*(lasty-nexty));
                double vector[] = new double[3];
                vector[0] = -nextx+lastx;
                vector[1] = -nexty+lasty;
                vector[2] = -nextz+lastz;
                double zDirection = vector[0]*cushion.globalShipDirectionVector[1] - vector[1]*cushion.globalShipDirectionVector[0];
                
                if (Math.abs(zDirection) < 0.01) {
                    addOrSubtract = 0;
                } else if (zDirection < 0) {
                    addOrSubtract = -1;
                } else {
                    addOrSubtract = 1;
                }
                
                if (distance < minDistance) {
                    wave_elevation_xy = waves.getInterpolatedWaveElevation(time,nextx,nexty) // pos.down
                            + pressureWC; // pos. down
                    submergence = -nextz+wave_elevation_xy; // Positive means above water
                    sectionArea += addOrSubtract * distance * (submergence+previousSubmergence)/2;

                } else if (distance > minDistance) {
                    double steps = Math.ceil(distance/minDistance);
                    double xStep = (nextx-lastx)/steps;
                    double yStep = (nexty-lasty)/steps;
                    double zStep = (nextz-lastz)/steps;
                    
                    for (int i=1; i<=steps; i++) {
                        nextx = lastx+xStep*i;
                        nexty = lasty+yStep*i;
                        nextz = lastz+zStep*i; 
                        wave_elevation_xy = waves.getInterpolatedWaveElevation(time,nextx,nexty) // pos.down
                                + pressureWC; // pos. down
                        submergence = -nextz+wave_elevation_xy; // Positive means above water
                        sectionArea += addOrSubtract * Math.sqrt(xStep*xStep+yStep*yStep) * (submergence+previousSubmergence)/2;
                        previousSubmergence = submergence;
                    }
                }
                
                if (nextCoord == cushion.coordNumber[secNr]-1)
                    goon = false;
                lastx = nextx;
                lasty = nexty;
                lastz = nextz;
                previousSubmergence = submergence;
                nextCoord++;   
            }
            
            // Contribution from section to total volume
            airCushionVolume += sectionArea*cushion.globaldL[secNr];
        }
        
        // Volume saturation: 1) Non negative. 2) How much the volume can change per iteration (each 0.05 sec)
        if (airCushionVolume <1 ){
            airCushionVolume = 1;
        }
        else if(airCushionVolume < 880){ // 880
            // All volume values above 900[m^3] means that the vessel is flying above water.
            // It can be helpful to drop the vessel from the air to observe behaviour
            // But no volume saturation for values above 875[m^3]
            if (airCushionVolume > prevAirCushionVolume + volumeSaterationPerIteration){
                airCushionVolume = prevAirCushionVolume + volumeSaterationPerIteration;
            }
            else if(airCushionVolume < prevAirCushionVolume - volumeSaterationPerIteration){
                airCushionVolume = prevAirCushionVolume - volumeSaterationPerIteration;
            }
        prevAirCushionVolume= airCushionVolume;
        }
    }
    
    
    public void calculateStatic(AirCushionGeometry cushion, double p_c, double draught, Constants constant) {
        this.airCushionVolume = 0;
        double pressureWC = p_c/(constant.rho*constant.g);
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++){
            double sectionArea = 0;

            boolean goon = true;
            
            int nextCoord = cushion.firstCoordInAirStarb[secNr];
            double nexty = cushion.crossyCoordLocalStarB[secNr];
            double nextz = cushion.crosszCoordLocalStarB[secNr];
            double lasty = nexty;
            double lastz = nextz;
            
            double previousSubmergence = lastz - draught + pressureWC;                     
            while (goon) {            

                nexty = cushion.localyCoord[secNr][nextCoord];
                nextz = cushion.localzCoord[secNr][nextCoord];
                nextCoord++;
                if (nextCoord == cushion.coordNumber[secNr])
                    goon = false;
                
                double submergence = nextz - draught + pressureWC;
                sectionArea += Math.abs(lasty-nexty)*(submergence + previousSubmergence)/2;
                lasty = nexty;
                lastz = nextz;
                previousSubmergence = submergence;
            }
            sectionArea = sectionArea*2; // Initial condition: Perfect symmetry        
            
            this.airCushionVolume += sectionArea*cushion.dx[secNr]; 
            
            // Initially just setting previously volume equal to current
            prevAirCushionVolume = airCushionVolume;
        }
    } 
    
    public double getCushionVolume(){
        return this.airCushionVolume;
    }
}