package no.marintek.simulator.ses;
import no.marintek.simulator.generic.GenericFederate;
import java.io.*;
import java.util.*;

/**
 * Fan. Slår den opp i tabellen eller lager en cont. modell av de få punktene som er der?. Todo
 * 
 * All air flows are from singel fans. Only when sent out to the CSI bus will one count for the total number of fans.
 * 
 * 
 * @author Øyvind Auestad
 */

public class Fan extends GenericFederate {
    
    private boolean initialized = false;
    private int points_on_curve = 0;
    private double[] Q_fan;
    private double[] p_fan; 
    private double Q_in, Q_0;
    private String fan_path;
    private double p_0; 
    private double p_c;
    private double g = 9.81, rho = 1025, time=0; // g: gravity, rho: mass density of sea water
    private boolean update_air_cushion = true;
    private int number_of_fans = 0;
    private int controlled_fan_pos;
    private int fan_pos;
    private double open_close_actuator_delay; // delay while going from closed to open fan
    private int gap = 0;
    private double controlled_fan_pos_arrival_time = 0;
    private double Q_inChar = 0;
    
   
    
    public Fan(String configFile, String simType){
        super(configFile, simType);
        logger.info("Starting simulation");
        System.out.println("config1");
        simulate();
    }
    
    public Fan(String configFile, String fedName, String simType){
        super(configFile, fedName, simType);
        logger.info("Starting simulation");
        System.out.println("config2");
        simulate();
    }
    
    private void simulate(){
        simApp.startSimulation();

        // Pre loop:
        this.fan_path = simApp.getModelProperties("fan_path",simApp.getFederateName());
        this.ReadFile(fan_path);
       
        while (simApp.isConnected()){
            performStep();
        }
        simApp.stopSimulation();
    }
    
     private void performStep(){ 
         //Attributes read every iteration:
        update_air_cushion = simApp.getReceivedObjectAttribute("update_air_cushion", "Case1_1.AirCushion").getValueAsBoolean();
        time = simApp.getReceivedObjectAttribute("time", "SimulationTime").getValueAsDouble();
        p_c = simApp.getReceivedObjectAttribute("p_c","Case1_1.AirCushion").getValueAsDouble();
        controlled_fan_pos = simApp.getReceivedObjectAttribute("controlled_fan_pos", "Case1_1.HeaveControl").getValueAsInteger();
        // False forste gang: todo
        
        // Update system
//        if (update_air_cushion || !initialized){
 //           updateFanSystem();
 //       }
        
        this.p_0 = simApp.getReceivedObjectAttribute("p_0","Case1_1.AirCushion").getValueAsDouble();
        this.Q_0 = getFanCharacteristicQ_in(p_0);
        this.number_of_fans = simApp.getOwnParameter("number_of_fans").getValueAsInteger();
        this.open_close_actuator_delay  = simApp.getOwnParameter("open_close_actuator_delay", "Case1_1.Fan").getValueAsDouble(); // Delay: sec
        
        // Send Q_in from fan characteristic graph to the Ride System
        // So the controller know what different control area avalible which is:
        // u e [0 , getFanCharacteristicQ_in(p_c)]
        
        
        
        
        if (actuatorInWantedPosition(open_close_actuator_delay, time)){
            this.fan_pos = controlled_fan_pos;
        }
        
        Q_in = (Q_inChar * fan_pos) / 100;
        Q_inChar = getFanCharacteristicQ_in(p_c);
        // Rewriting: fan_pos = 100*(Q_in)/Q_inChar
        //Q_in = (Q_inChar/100) * (fan_pos);

        //Writes out actual fan position (Physicly the position of the cone, see master thesis)
        // System.out.println("Fan position wanted by controller = "+controlled_fan_pos + "--- Actual fan position = "+fan_pos);
        simApp.getOwnAttribute("fan_pos").setValueAsDouble(fan_pos);
        
        // Get total inflow from all fans:
        simApp.getOwnAttribute("Q_in_characteristic").setValueAsDouble(Q_inChar);
        simApp.getOwnAttribute("Q_in").setValueAsDouble(Q_in*number_of_fans);
        simApp.getOwnAttribute("Q_0").setValueAsDouble(Q_0*number_of_fans);
        simApp.performStep();  
        
     }
     
     private void updateFanSystem(){
            this.p_0 = simApp.getReceivedObjectAttribute("p_0","Case1_1.AirCushion").getValueAsDouble();
            this.Q_0 = getFanCharacteristicQ_in(p_0);
            
            this.number_of_fans = simApp.getOwnParameter("number_of_fans").getValueAsInteger();
            this.open_close_actuator_delay  = simApp.getOwnParameter("open_close_actuator_delay", "Case1_1.Fan").getValueAsDouble(); // Delay: sec
            // IN order to calculate correct Q_0 from the fan characteristic we 
            // must know in which position the fan is while receiving p_0
            // (while we receive p_0 the control system has 50% effect from louvers and fan
            // so that the vessel will control around its equlibrium point)

     }
     
    // Actuator air flow rate delay:
    private boolean actuatorInWantedPosition(double open_close_actuator_delay, double time){
        // Tells if handle is in the current wanted pos (from RideSystem)
        if(gap*(open_close_actuator_delay/100) + controlled_fan_pos_arrival_time <= time ){
            // In position now!
            controlled_fan_pos_arrival_time = time;
            gap = Math.abs(fan_pos - controlled_fan_pos);
            return true;
        }else{
            // Not in position!
            return false;
        }
    }
     
     private void ReadFile(String filename) {
        System.out.println("SESFederate: Reading fan characteristics file: " + filename);
        try {
            
            BufferedReader input = new BufferedReader(new FileReader(filename));
            
            // Read toptext. This text is not of interest
            try{
                for (int i=0; i<4; i++){
                    String dummy = input.readLine();
                }
            } catch (EOFException eof){}            
            
            // Read number of data set in file
            String dummy = input.readLine();
            points_on_curve = Integer.parseInt(dummy);
            
            // Initialize arrays
            Q_fan = new double[points_on_curve];
            p_fan = new double[points_on_curve];
                       
            dummy = input.readLine(); // Line of no interest
            dummy = input.readLine(); // Line of no interest
     
            for (int i=0; i<points_on_curve; i++){
                dummy = input.readLine(); dummy = dummy.trim();
                {
                    StringTokenizer dummyToken = new StringTokenizer(dummy);
                    Q_fan[i] = Double.valueOf(dummyToken.nextToken()).doubleValue();
                    p_fan[i] = Double.valueOf(dummyToken.nextToken()).doubleValue();
                }
            }          
        } catch (IOException e) {
            System.err.println("Error: " + e.toString());
        }        
    }  
     
    private double getFanCharacteristicQ_in(double p_c) {
            double Q_in = 0;
            double p_waterColumn = p_c*1000/(g*rho);    // Air cushion pressure given in mmWC in fan characteristics

            int i = 0;
            if (p_fan[this.points_on_curve-1] >= p_waterColumn) { // Pressure in cushion too low for characterisitcs (negative), Q is set to corresponding limit (In principal Q_in = max)
                Q_in = Q_fan[this.points_on_curve-1];
            } else if (p_fan[0] <= p_waterColumn) { // Pressure in cushion too high for characterisitcs, Q is set to corresponding limit (In principal Q_in = 0)
                Q_in = Q_fan[0];
            } else {
                while(true) {
                    i++;
                    if (p_fan[i]<p_waterColumn) // The actual pressure correspond to closest table value
                        break;
                }
                // Interpolate to find air flow
                Q_in = Q_fan[i] + 
                        (Q_fan[i-1]-Q_fan[i]) * (p_waterColumn-p_fan[i]) / 
                        (p_fan[i-1]-p_fan[i]); 
            }
        return Q_in;
    }

    public static void main(String[] args){
        // This class needs input from the command line to start correctly
        String federate = null;         // First input argument
        String simBus = null;           // Second input argument
        String federationFile = null;   // Third input argument
               
        if(args.length > 1){
            federationFile = args[0];
            if(args[1].equalsIgnoreCase("csi") || args[1].equalsIgnoreCase("hla")){
                simBus = args[1];
                System.out.println(simBus);
                Fan fan = new Fan(federationFile, simBus);
            } else {
                federate = args[1];
            }
        } 
        else{
            System.out.println("Federation file and sim bus type must be specified");
        }
        
        if(args.length > 2){
            federate = args[1];
            simBus = args[2];
            Fan fan = new Fan(federationFile,federate, simBus);
        }
    }
}