/*
 * InitialSESData
 * Program made to due static calculations
 * 
 * Made by Trygve Espeland Halvorsen, Spring 2008
 */

package no.marintek.simulator.ses;

public class InitialSESData {
    // Q_0 hardcoded only for very first iteration 
    // an`t receive anything from the CSI bus until third iteration (t = 0.15s)
    private double Q_0 = 120; // TODO, value wont even be used with the new control system.
    private double wet_surface_0 = 0;
    private double cushion_volume_0 = 0;
    private double A_Leakage_Stat_Calc = 0;
    private double A_Leakage_Stat = 0;
    private double lift_ratio_0 = 0;
    public double force_surge = 0;
    public double force_sway = 0;
    public double force_heave = 0;
    public double moment_roll = 0;
    public double moment_pitch = 0;
    public double moment_yaw = 0;

    public InitialSESData(SESHull hull, AirCushionGeometry cushion, Seal sealStern, Seal sealBow, double p_0, double draught, 
           SESMass mass, Constants constant, CushionVolume volume, SurfaceEffectShipSeals SESseal) {

        // Calculations
        hull.updateStatic(constant, draught);
        cushion.updateStatic(constant, p_0, draught);
        WetHull wetSurface = new WetHull(hull, cushion);
       // CushionVolume volume = new CushionVolume(cushion, p_0, draught, constant);
        volume.calculateStatic(cushion, p_0, draught, constant);
        //volume = new CushionVolume(cushion, p_0, draught, constant);
        CushionLeakage leakage = new CushionLeakage(cushion, hull, p_0, constant, draught);
        
        SESForces initForce = new SESForces(cushion, p_0);
 
        //mass.massAirCushion = initForce.getCushionHeave()/constant.g;
        mass.massAirCushion = initForce.cforce_heave/constant.g;
        mass.massDisplacement = mass.massTotal-mass.massAirCushion;
        this.lift_ratio_0 = mass.massAirCushion*100/mass.massTotal;
        
        // Static calculations:
        this.wet_surface_0 = wetSurface.wetArea;
        //this.cushion_volume_0 = volume.getCushionVolume();
        this.cushion_volume_0 = 635.538;
        // Static leakage area, corresponds to Q_0, static air flow:
        this.A_Leakage_Stat = (Q_0)*Math.sqrt(constant.rho_a/(2*p_0));        
        
        // Static leakage area, as calculated from geometry:
        this.A_Leakage_Stat_Calc = Math.min(leakage.totalPassiveLeakage, A_Leakage_Stat);  
                
        this.force_surge = initForce.cforce_surge;
        this.force_sway = initForce.cforce_sway;
        this.force_heave = initForce.cforce_heave;
        this.moment_roll = initForce.cmoment_roll;
        this.moment_pitch = initForce.cmoment_pitch;
        this.moment_yaw = initForce.cmoment_yaw;
        
        
        // If calculated leakage area is larger than that corresponding to Q_0, the vessel is unstable in static condition
        // Therefore: Write warning message to command window 
        if (leakage.totalPassiveLeakage > this.A_Leakage_Stat) //
            System.out.println("SESFederate: Warning: Error in static condition, static leakage area is too large");
    } 
    
    public double getA_Leakage_Stat_Calc() { return A_Leakage_Stat_Calc; }
    public double getwet_surface_0() { return wet_surface_0; }
    public double getcushion_volume_0() { return cushion_volume_0; } 
    public double getQ_0() { return Q_0; }
    public double getA_Leakage_Stat() { return A_Leakage_Stat; }
    public double getLift_ratio_0() { return lift_ratio_0; }
    public double getQ_in0(){return Q_0;}
    
}