/*
 * This Kalmanfilter is based on measuring the position and only position
 * 
 */
package no.marintek.simulator.ses;

import org.jblas.DoubleMatrix;

/**
 *
 * @author Umoe
 */
public class KalmanFilter {
    private DoubleMatrix x_pred; // predicted states (pos and velocity)           = Q
   // private DoubleMatrix x_meas; // measured states                              =Q_loc_mes
    private DoubleMatrix x_est; // estimated states                              = Q_estimate
    private double x_est_pos;
    private double z;             // Actual measure from MRU
    private double u;
    private DoubleMatrix A;
    private DoubleMatrix B;
    private DoubleMatrix C;
       
    private double stdv_x, stdv_z;  // Process, measurement noise
    // Process variance
    private DoubleMatrix Ex; 
    // Measurement variance
    private double Ez;
    
    // Estimate of position variance (Co-Variance matrix)
    // Estimate the merge of process and measurement variance ?????
    private DoubleMatrix P_pred; 
    private DoubleMatrix P_est; 
    // Kalman gain (2x1):
    private DoubleMatrix K; 
    
    private DoubleMatrix I = DoubleMatrix.eye(2);
    
    private double heave_ref;
    
    public KalmanFilter(double dt, double heave_ref){

        this.heave_ref = heave_ref;
            
        // Define the Equation of Motion (see Kalmanfilter in the report)
        // In matlab A is on the form A = [1 dt; 0 1;]. As the syntax below indicates
                
        // Define update equations (Coefficient matrices from physical model)
        A = new DoubleMatrix(new double[][]{
                            {1.0,  dt},
                            {0,    1.0} });
       
        // B is a column vector (2x1)
        B = new DoubleMatrix(new double[]
                            {0.5*Math.pow(dt, 2),   dt});
        
        // C is a row vector (1x2)
        C = new DoubleMatrix(new double[]
                            {1, 0}).transpose();
        
        // Define main variabels (state and input)
        x_pred = new DoubleMatrix(new double[]{this.heave_ref, 0});
        x_est = x_pred;
        u = 0;
        stdv_x = 0.1; // Process noise
        stdv_z = 10;  // Measurement noise 7
        Ez = Math.pow(stdv_z,2);
        Ex = new DoubleMatrix(new double[][]{
                            {B.get(0)*B.get(0),   B.get(0)*B.get(1)},
                            {B.get(1)*B.get(0),   B.get(1)*B.get(1) } });
        Ex = Ex.mmul(Math.pow(stdv_x,2)); // Ex = (stdv_x)^2 * Ex

        // Inserting offline calibration of P,S,L and K matrices:
        // P_pred =Ex; // Init co-var. matrix.
        P_pred = new DoubleMatrix(new double[][]{
                            {0.179382,  0.02005},
                            {0.020168,  0.00448} });
        P_est = new DoubleMatrix(new double[][]{
                            {0.177381,  0.019832},
                            {0.019944,  0.004459} });  
        
        K = new DoubleMatrix(new double[]
                            {0.01114937,   0.0012465});

    }
    
   public void offlineParameterCalibration(double stdv_x, double stdv_z, double K_p_louver, double K_p_fan){
       // AFTER RUNNDING PARAMETERCALIBRATION (UPDATE P, K, L, S).
       // ONE MUST UPDATE THE INITAL respectively matrices in the construtor
       this.stdv_x = stdv_x;
       this.stdv_z = stdv_z;

       // Get input paramters (for tuning)
        Ez = Math.pow(stdv_z,2);
        Ex = new DoubleMatrix(new double[][]{
                            {B.get(0)*B.get(0),   B.get(0)*B.get(1)},
                            {B.get(1)*B.get(0),   B.get(1)*B.get(1) } });
        Ex = Ex.mmul(Math.pow(stdv_x,2)); // Ex = (stdv_x)^2 * Ex
        
        
        
        // P_predicted = APA´ + Ex
        P_pred = A.mmul(P_est).mmul(A.transpose()).add(Ex);
        
        // K = P_pred * C` * (C*P_pred*C + Ex)^-1 :
        DoubleMatrix tmp1Matrix = C.mmul(P_pred).mmul(C.transpose()).add(Ez); // (1x1)
        double tmp1 = Math.pow(tmp1Matrix.get(0),-1);
        K = A.mmul(P_pred).mmul(C.transpose()).mmul(tmp1);

        //P_est = (I - KC)P_pred
        P_est = (I.sub(K.mmul(C))).mmul(P_pred);
   }
    
    public void onlineStateEstimation(double Kp, double A_L_Louver_Max, double Q_in_characteristic, double u){
        // P, S, L, K is calculated offline and is estimated to the values given 
        // in the constructor.

        //x_predicted = A*x_est + B*u
        x_pred = (A.mmul(x_est)).add(B.mmul(u));
        DoubleMatrix z_pred = C.mmul(x_pred); // z_predM is a 1x1 Matrix...
        x_est = x_pred.add(K.mmul(z-z_pred.get(0)));
    }
    
    public void getSensorReadings(double mru_down){
        this.z = mru_down;
    }

    public double getEstimatedPos(){ 
        // Estimated heave position
        return x_est.get(0);
    }
    
    public double getEstimatedVel(){ 
        return x_est.get(1);
    }

}
