/*
 * Receives current Z value (CG FROM NED) attributes from the CSI.
 * 
 * This values is controlled with a simple P-Controller
 * The output is wanted leak area.
 */
package no.marintek.simulator.ses;

public class RideController {

    // Wanted position of the Z coordinate of global coordinates (Z value of cg)
    
    private double u_louver;
     private double u_fan;
    private double e;
    private double A_L_Louver_Max;

    private double pos_heave_REF;
    private double K_p_louver;
    private double K_p_fan; 
    
    private double Q_in_characteristic;
   
   // private double A_L_Louver_ctrl; delete

    // Change the actuator resolution (20 steps from closed to open actuator)
    private int resolutionFan = 20;
    private int resolutionLouver = 20;
    
    
    public RideController(double K_p_fan, double K_p_louver, double pos_heave_REF, double A_L_Louver_Max, double Q_in_characteristic){
        this.pos_heave_REF = pos_heave_REF;
        this.K_p_louver = K_p_louver;
        this.K_p_fan = K_p_fan;
        this.A_L_Louver_Max = A_L_Louver_Max;
        this.Q_in_characteristic = Q_in_characteristic;
    }
    
    public void calculate(double pos_heave, double u, double K_p_louver, double K_p_fan){
        //e = pos_heave_REF - pos_heave;
        //e = u;
        double K_kalman = 0.8;
        u_louver = K_kalman * K_p_louver * u;
        u_fan = - K_kalman * K_p_fan * u;
        
        /*
        double K_p_kalman = 10;
      //  System.out.println("error controller = "+e);
        u_louver = K_p_kalman * K_p_louver*e;
        u_fan =  - K_p_kalman * K_p_fan*e;
         * 
         */
    }
    
    public int getLouverPos(){
        // A scaled value between 0 and 100. 0: no louver leakage, 100: max louver leak area
        
        // Saturation. The leak cant be bigger than the total louver area specified
        // in LouverFederate. Leakage area also is > 0. The saturation is:
        // A_L_Louver_Max >= u_louver + A_L_Louver_Max/2 >= 0
        if (u_louver <= -A_L_Louver_Max/2){
            u_louver = - A_L_Louver_Max/2;
        }
        else if(u_louver >= A_L_Louver_Max/2){
            u_louver = A_L_Louver_Max/2;
        }
        
        // Wanted area for the louvers in order to control heave movement
        // The louver area leakage that is used for cushion calculataion (in SESFederate)
        // will have a actuator delay which is implemented in FanFederate
        double A_L_Louver_ctrl = u_louver + A_L_Louver_Max/2;
        // Louver openess in percentage. If 0: louver is closed, if 100: its 100% open
        int controlled_louver_pos = (int)(100 * (A_L_Louver_ctrl/A_L_Louver_Max));
        // Round off to actuator resolution (example: 20 steps from closed to open)
        // With 20 steps: Function will return a number from 0 to 100 rounded iff to closest 5.
        return (int)(Math.round(controlled_louver_pos *resolutionLouver/100.0) * (100.0/resolutionLouver));
    }
    
    public int getFanPos(){
        // A scaled integer from 0 and 100. 0: no fan contribution, 100: max contr.
        // Saturation: Q_in_char >= u + Q_in_char/2 >= 0
        
        if (u_fan <= -Q_in_characteristic/2){
            u_fan = -Q_in_characteristic/2;
        }
        else if(u_fan >= Q_in_characteristic/2){
             u_fan = Q_in_characteristic/2;
        }
        
        // Air flow from fans which is the controll value. The desired inflow from the fans.
        double Q_in_ctrl = u_fan + Q_in_characteristic/2;
        
        int controlled_fan_pos = (int)(100 * (Q_in_ctrl/Q_in_characteristic));
       
        // Rounds off controlled_fan_pos to closest (100/resolutionFan)
        return (int)(Math.round(controlled_fan_pos *resolutionFan/100.0) * (100.0/resolutionFan));
    }
    
}