/*
 * wetSurface
 * Calculates wet surface on the hull
 * Used to obtain viscous resistance
 * 
 * Made by Trygve Espeland Halvorsen, Spring 2008
 */

package no.marintek.simulator.ses;


public class WetHull {
    public double wetArea = 0;

    
    public WetHull(SESHull hull, AirCushionGeometry cushion) {
        this.wetArea = 0;
        wetArea += this.wetHull(hull);
        wetArea += this.wetCushion(cushion);
    }
    
    
    private double wetHull(SESHull hull) {
        double wetHullArea = 0;
        
        // ----- Starboard side -----
        for (int secNr=0; secNr<hull.sectionNumber; secNr++) {

            for (int elemNr=hull.firstWetCoordStarb[secNr]; elemNr<hull.lowestCoordNr[secNr]; elemNr++) {
                wetHullArea += hull.elementLength[secNr][elemNr]*hull.dx[secNr];
            }
            
            wetHullArea += hull.firstWetLengthStarb[secNr]*hull.dx[secNr];
        }
        
        // ----- APort side -----
        for (int secNr=0; secNr<hull.sectionNumber; secNr++) {

            for (int elemNr=hull.firstWetCoordAPort[secNr]; elemNr<hull.lowestCoordNr[secNr]; elemNr++) {
                wetHullArea += hull.elementLength[secNr][elemNr]*hull.dx[secNr];
            }
            
            wetHullArea += hull.firstWetLengthAPort[secNr]*hull.dx[secNr];
        }
        return wetHullArea;
    }
    
    
    private double wetCushion(AirCushionGeometry cushion) {
        double wetCushionArea = 0;
        
        // ----- Starboard side -----
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++) {
            
            for (int elemNr=0; elemNr<cushion.firstCoordInAirStarb[secNr]-1; elemNr++) {
                wetCushionArea += cushion.elementLength[secNr][elemNr]*cushion.dx[secNr];
            }
            
            wetCushionArea += cushion.firstWetLengthStarb[secNr]*cushion.dx[secNr];
        }
        
        // ----- APort side -----
        for (int secNr=0; secNr<cushion.sectionNumber; secNr++) {
            
            for (int elemNr=0; elemNr<cushion.firstCoordInAirAPort[secNr]-1; elemNr++) {
                wetCushionArea += cushion.elementLength[secNr][elemNr]*cushion.dx[secNr];
            }
            
            wetCushionArea += cushion.firstWetLengthAPort[secNr]*cushion.dx[secNr];
        }
        return wetCushionArea;
    }
}
